@@ -5,6 +5,7 @@ use std::sync::RwLock;
 
 use anyhow::{Context, Result};
 use bio::stats::{bayesian, LogProb};
+use bio_types::genome;
 use derive_builder::Builder;
 use itertools::Itertools;
 use progress_logger::ProgressLogger;
@@ -21,15 +22,17 @@ use crate::errors;
 use crate::grammar;
 use crate::utils;
 use crate::variants::evidence::observation::{Observation, ReadPosition};
-use crate::variants::model;
 use crate::variants::model::modes::generic::LikelihoodOperands;
 use crate::variants::model::modes::generic::{
     self, GenericLikelihood, GenericModelBuilder, GenericPosterior,
 };
 use crate::variants::model::Contamination;
-use crate::variants::model::{bias::Biases, AlleleFreq};
+use crate::variants::model::{self};
+use crate::variants::model::{bias::Artifacts, AlleleFreq};
 use crate::variants::types::breakends::BreakendIndex;
 
+use super::preprocessing::Observations;
+
 pub(crate) type AlleleFreqCombination = LikelihoodOperands;
 
 pub(crate) type Model<Pr> =
@@ -47,8 +50,7 @@ where
     omit_read_orientation_bias: bool,
     omit_read_position_bias: bool,
     omit_softclip_bias: bool,
-    omit_divindel_bias: bool,
-    min_divindel_other_rate: f64,
+    omit_homopolymer_artifact_detection: bool,
     scenario: grammar::Scenario,
     outbcf: Option<PathBuf>,
     contaminations: grammar::SampleInfo<Option<Contamination>>,
@@ -175,11 +177,11 @@ where
               event (PROB_ARTIFACT).\">",
         );
         header.push_record(
-            b"##FORMAT=<ID=DIB,Number=A,Type=String,\
-              Description=\"Divindel bias estimate: * indicates that ALT allele is associated with \
-              with indel operations of varying length, . indicates that there is no divindel bias.
-              Divindel bias is indicative of systematic PCR amplification errors, e.g. induced by \
-              homopolymers. Probability for divindel bias is captured by the ARTIFACT \
+            b"##FORMAT=<ID=HE,Number=A,Type=String,\
+              Description=\"Homopolymer error estimate: * indicates that ALT allele is associated with \
+              with homopolymer indel operations of varying length, . indicates that there is no homopolymer error.
+              Homopolymer error is indicative of systematic PCR amplification errors. \
+              Probability for such homopolymer artifacts is captured by the ARTIFACT \
               event (PROB_ARTIFACT).\">",
         );
 
@@ -294,6 +296,10 @@ where
                 }
             }
 
+            // obtain variant type
+            let variant_type =
+                utils::collect_variants(records.first_not_none_mut()?, false, None)?[0].to_type();
+
             let mut work_item = self.preprocess_record(&mut records, i, &observations)?;
 
             // process work item
@@ -305,6 +311,7 @@ where
                 work_item.check_read_orientation_bias,
                 work_item.check_read_position_bias,
                 work_item.check_softclip_bias,
+                work_item.check_homopolymer_artifact_detection,
             );
             _model = models.entry(model_mode).or_insert_with(|| self.model());
             {
@@ -313,10 +320,6 @@ where
                 *entry = Some(work_item.rid);
             }
 
-            // obtain variant type
-            let variant_type =
-                utils::collect_variants(records.first_not_none_mut()?, false, None)?[0].to_type();
-
             self.configure_model(
                 work_item.rid,
                 _last_rid,
@@ -328,7 +331,7 @@ where
                 work_item.check_strand_bias,
                 work_item.check_read_position_bias,
                 work_item.check_softclip_bias,
-                work_item.check_divindel_bias,
+                work_item.check_homopolymer_artifact_detection,
             )?;
 
             self.call_record(&mut work_item, _model, &events);
@@ -351,6 +354,8 @@ where
             let start = first_record.pos() as u64;
             let chrom = chrom(observations.first_not_none()?, first_record);
 
+            let _locus = genome::Locus::new(str::from_utf8(chrom).unwrap().to_owned(), start);
+
             let call = CallBuilder::default()
                 .chrom(chrom.to_owned())
                 .pos(start)
@@ -414,7 +419,7 @@ where
             check_strand_bias: !self.omit_strand_bias,
             check_read_position_bias: is_snv_or_mnv && !self.omit_read_position_bias,
             check_softclip_bias: is_snv_or_mnv && !self.omit_softclip_bias,
-            check_divindel_bias: !self.omit_divindel_bias,
+            check_homopolymer_artifact_detection: false,
         };
 
         if let Some(ref event) = work_item.bnd_event {
@@ -429,10 +434,20 @@ where
         let mut pileups = Vec::new();
         for record in records.iter_mut() {
             let pileup = if let Some(record) = record {
-                let mut pileup = read_observations(record)?;
+                let Observations {
+                    observations: mut pileup,
+                    is_homopolymer_indel,
+                } = read_observations(record)?;
+                if is_homopolymer_indel && !self.omit_homopolymer_artifact_detection {
+                    // METHOD: check for homopolymer artifacts if at least one pileup contains the corresponding information.
+                    work_item.check_homopolymer_artifact_detection |= true;
+                }
                 if is_snv_or_mnv {
                     // METHOD: adjust MAPQ to get rid of stochastically inflated ones
-                    //Observation::adjust_prob_mapping(&mut pileup);
+                    // This takes the arithmetic mean of all MAPQs in the pileup.
+                    // By that, we effectively diminish high MAPQs of reads that just achieve them
+                    // because of e.g. randomly better matching bases in themselves or their mates.
+                    Observation::adjust_prob_mapping(&mut pileup);
                     // METHOD: remove non-standard alignments. They might come from near
                     // SVs and can induce artifactual SNVs or MNVs. By removing them,
                     // we just conservatively reduce the coverage to those which are
@@ -467,7 +482,7 @@ where
         consider_strand_bias: bool,
         consider_read_position_bias: bool,
         consider_softclip_bias: bool,
-        consider_divindel_bias: bool,
+        consider_homopolymer_error: bool,
     ) -> Result<()> {
         if !rid.map_or(false, |rid: u32| current_rid == rid) {
             // rid is not the same as before, obtain event universe
@@ -478,7 +493,7 @@ where
             events.push(model::Event {
                 name: "absent".to_owned(),
                 vafs: grammar::VAFTree::absent(self.n_samples()),
-                biases: vec![Biases::none()],
+                biases: vec![Artifacts::none()],
             });
 
             // add events from scenario
@@ -486,16 +501,15 @@ where
                 events.push(model::Event {
                     name: event_name.clone(),
                     vafs: vaftree.clone(),
-                    biases: vec![Biases::none()],
+                    biases: vec![Artifacts::none()],
                 });
 
-                let biases: Vec<_> = Biases::all_artifact_combinations(
+                let biases: Vec<_> = Artifacts::all_artifact_combinations(
                     consider_read_orientation_bias,
                     consider_strand_bias,
                     consider_read_position_bias,
                     consider_softclip_bias,
-                    consider_divindel_bias,
-                    self.min_divindel_other_rate,
+                    consider_homopolymer_error,
                 )
                 .collect();
                 if !biases.is_empty() {
@@ -646,7 +660,7 @@ where
         for (map_estimates, _) in model_instance.event_posteriors() {
             if map_estimates
                 .iter()
-                .any(|estimate| estimate.biases.is_artifact())
+                .any(|estimate| estimate.artifacts.is_artifact())
                 && !is_artifact
             {
                 // METHOD: skip MAP that is an artifact if the overall artifact event is not the strongest one.
@@ -661,15 +675,17 @@ where
                     let mut sample_builder = SampleInfoBuilder::default();
                     sample_builder.observations(pileup);
                     match estimate {
-                        model::likelihood::Event { biases, .. } if biases.is_artifact() => {
+                        model::likelihood::Event {
+                            artifacts: biases, ..
+                        } if biases.is_artifact() => {
                             sample_builder
                                 .allelefreq_estimate(AlleleFreq(0.0))
-                                .biases(biases.clone());
+                                .artifacts(biases.clone());
                         }
                         model::likelihood::Event { allele_freq, .. } => {
                             sample_builder
                                 .allelefreq_estimate(*allele_freq)
-                                .biases(Biases::none());
+                                .artifacts(Artifacts::none());
                         }
                     };
                     Some(sample_builder.build().unwrap())
@@ -700,5 +716,5 @@ struct WorkItem {
     check_strand_bias: bool,
     check_read_position_bias: bool,
     check_softclip_bias: bool,
-    check_divindel_bias: bool,
+    check_homopolymer_artifact_detection: bool,
 }
