@@ -10,8 +10,8 @@ use std::fmt::Debug;
 use bio::alignment::AlignmentOperation;
 use bio::pattern_matching::myers::{self, long};
 use bio::stats::pairhmm;
-use itertools::Itertools;
 
+use crate::utils::homopolymers::HomopolymerIndelOperation;
 use crate::variants::evidence::realignment::pairhmm::{RefBaseEmission, EDIT_BAND};
 
 enum Myers {
@@ -21,7 +21,7 @@ enum Myers {
 
 pub(crate) struct EditDistanceCalculation {
     myers: Myers,
-    read_seq_len: usize,
+    read_seq: Vec<u8>,
 }
 
 impl EditDistanceCalculation {
@@ -38,17 +38,15 @@ impl EditDistanceCalculation {
         P: Iterator<Item = u8> + DoubleEndedIterator + ExactSizeIterator,
     {
         let l = read_seq.len();
+        let read_seq = read_seq.collect();
 
         let myers = if l <= 128 {
-            Myers::Short(myers::Myers::new(read_seq))
+            Myers::Short(myers::Myers::new(&read_seq))
         } else {
-            Myers::Long(long::Myers::new(read_seq))
+            Myers::Long(long::Myers::new(&read_seq))
         };
 
-        EditDistanceCalculation {
-            myers,
-            read_seq_len: l,
-        }
+        EditDistanceCalculation { myers, read_seq }
     }
 
     /// Returns a reasonable upper bound for the edit distance in order to band the pairHMM computation.
@@ -58,12 +56,13 @@ impl EditDistanceCalculation {
         emission_params: &E,
         max_dist: Option<usize>,
     ) -> Option<EditDistanceHit> {
-        let ref_seq =
-            (0..emission_params.len_x()).map(|i| emission_params.ref_base(i).to_ascii_uppercase());
+        let ref_seq = || {
+            (0..emission_params.len_x()).map(|i| emission_params.ref_base(i).to_ascii_uppercase())
+        };
         let mut best_dist = usize::max_value();
         let mut positions = Vec::new();
         let alignments: Vec<Alignment>;
-        let max_dist = max_dist.unwrap_or(self.read_seq_len);
+        let max_dist = max_dist.unwrap_or(self.read_seq.len());
 
         let mut handle_match = |pos, dist: usize| match dist.cmp(&best_dist) {
             Ordering::Less => {
@@ -79,7 +78,7 @@ impl EditDistanceCalculation {
 
         match &mut self.myers {
             Myers::Short(myers) => {
-                let mut matches = myers.find_all_lazy(ref_seq, max_dist as u8);
+                let mut matches = myers.find_all_lazy(ref_seq(), max_dist as u8);
                 for (pos, dist) in &mut matches {
                     handle_match(pos, dist as usize);
                 }
@@ -97,7 +96,7 @@ impl EditDistanceCalculation {
                     .collect();
             }
             Myers::Long(myers) => {
-                let mut matches = myers.find_all_lazy(ref_seq, max_dist);
+                let mut matches = myers.find_all_lazy(ref_seq(), max_dist);
                 for (pos, dist) in &mut matches {
                     handle_match(pos, dist);
                 }
@@ -124,77 +123,54 @@ impl EditDistanceCalculation {
             let start = alignments[0].start();
             // take the last (aka first because we are mapping backwards) position for an upper bound of the putative end
             let end = cmp::min(
-                alignments.last().unwrap().start() + self.read_seq_len + best_dist as usize,
+                alignments.last().unwrap().start() + self.read_seq.len() + best_dist as usize,
                 emission_params.len_x(),
             );
 
-            // METHOD: obtain indel operations for divindel bias
-            let best_indel_operations = alignments
+            // METHOD: obtain indel operations for homopolymer error model
+            let homopolymer_indel_lens: Vec<_> = alignments
                 .iter()
-                .map(|alignment| {
-                    let mut ref_pos = emission_params.ref_offset() + alignment.start;
-
-                    // METHOD: group operations by indel or not, check whether they overlap the variant,
-                    // and report additional indels that remain in the variant range.
-                    // These are indicative of divindel bias (i.e. some wild disagreeing indel operations cause a caller
-                    // to interpet them as an indel variant, but in reality it is e.g. a PCR homopolymer error.)
-                    alignment
-                        .operations()
-                        .iter()
-                        .group_by(|op| {
-                            matches!(op, AlignmentOperation::Del | AlignmentOperation::Ins)
-                        })
-                        .into_iter()
-                        .filter_map(|(is_indel, group)| {
-                            let group: Vec<_> = group.collect();
-
-                            let mut group_ref_len = 0;
-                            for op in &group {
-                                // update ref_pos
-                                match op {
-                                    AlignmentOperation::Match
-                                    | AlignmentOperation::Subst
-                                    | AlignmentOperation::Del => group_ref_len += 1,
-                                    AlignmentOperation::Ins => (),
-                                    _ => unreachable!(),
-                                }
-                            }
-
-                            let ret = if let Some(variant_ref_range) =
-                                emission_params.variant_ref_range()
+                .filter_map(|alignment| {
+                    if let Some(operation) = HomopolymerIndelOperation::from_alignment(
+                        &ref_seq().skip(alignment.start).collect::<Vec<_>>(),
+                        &self.read_seq,
+                        &alignment.operations,
+                    ) {
+                        if let Some(variant_ref_range) = emission_params.variant_ref_range() {
+                            let ref_pos = emission_params.ref_offset()
+                                + alignment.start
+                                + operation.text_pos();
+                            // METHOD: check whether the operation is within the variant range.
+                            // In case of a deletion (operation.len() < 0) we also check whether the
+                            // end of the deletion is within the variant ref range.
+                            if variant_ref_range.contains(&ref_pos)
+                                && (operation.len() > 0
+                                    || variant_ref_range
+                                        .contains(&(ref_pos + operation.len().abs() as usize)))
                             {
-                                if is_indel
-                                    && (variant_ref_range.contains(&ref_pos)
-                                        || variant_ref_range.contains(&(ref_pos + group_ref_len))
-                                        || (variant_ref_range.start >= ref_pos
-                                            && variant_ref_range.end <= ref_pos + group_ref_len))
-                                {
-                                    // METHOD: indel ops that overlap with the variant interval are recorded here.
-                                    Some(group)
-                                } else {
-                                    None
-                                }
+                                Some(operation.len())
                             } else {
                                 None
-                            };
-
-                            ref_pos += group_ref_len;
-
-                            ret
-                        })
-                        .flatten()
-                        .cloned()
-                        .collect_vec()
+                            }
+                        } else {
+                            None
+                        }
+                    } else {
+                        None
+                    }
                 })
-                .min_by_key(|indels| indels.len())
-                .unwrap();
+                .collect();
+            let homopolymer_indel_len = homopolymer_indel_lens
+                .iter()
+                .min_by_key(|indel_len| *indel_len)
+                .cloned();
 
             Some(EditDistanceHit {
                 start,
                 end,
                 dist: best_dist,
                 alignments,
-                best_indel_operations,
+                homopolymer_indel_len,
             })
         }
     }
@@ -218,8 +194,8 @@ pub(crate) struct EditDistanceHit {
     dist: usize,
     #[getset(get = "pub(crate)")]
     alignments: Vec<Alignment>,
-    #[getset(get = "pub(crate)")]
-    best_indel_operations: Vec<AlignmentOperation>,
+    #[getset(get_copy = "pub(crate)")]
+    homopolymer_indel_len: Option<i8>,
 }
 
 impl EditDistanceHit {
